<?php
/**
 * @author     polkurier.pl <api@polkurier.pl>
 * @copyright  polkurier.pl
 * @license    https://opensource.org/licenses/OSL-3.0 OSL/OSL-3.0
 * @category   Module
 * @package    Presta 8
 * @subpackage polkurier
 * @link       https://www.polkurier.pl
 */

namespace Polkurier\Order;

use Polkurier\ApiMetod\ApiUtils;
use Polkurier\ApiMetod\StatusOrderApi;
use Polkurier\Model\PolkurierConfig;
use Tools;

final class OrderPolkurier
{
    public function __construct()
    {
    }

    public function ListOrders($page = 1, $limit = 100, $find = []): ?array
    {
        $order = OrdersModel::getOrderList($page, $limit, $find);

        if (!empty($order)) {

            foreach ($order as &$item) {

                $item['number_label'] = sprintf('<a href="%s">%s</a>', $item['traking_url'], $item['number_label']);
                $item = $this->UpdateStatusTraking($item);
                $item['receiver'] = self::SetReceiver($item);
                $item['url_presta_order'] = 'index.php?tab=AdminOrders&id_order=' . (int)$item['id_order_presta'] . '&vieworder' . '&token=' . Tools::getAdminTokenLite('AdminOrders');
            }

            return $order;
        }
        return null;
    }

    public function ListOrdersByPrestaId($id): ?array
    {
        $order = OrdersModel::getOrderListByOrderPresta($id);

        if (!empty($order)) {

            foreach ($order as &$item) {
                $item['number_label'] = sprintf('<a href="%s">%s</a>', $item['traking_url'], $item['number_label']);
                $item = $this->UpdateStatusTraking($item);
            }
            return $order;
        }
        return null;
    }

    public static function CountOrder(): int
    {
        return OrdersModel::getCountOrder();
    }

    private static function SetReceiver($item): string
    {
        $receiver = [];

        if (!empty($item['company'])) {
            $receiver[] = $item['company'];
        }
        if (!empty($item['first_name']) || !empty($item['last_name'])) {
            $receiver[] = $item['first_name'] . ' ' . $item['last_name'];
        }
        return implode(', ', $receiver);
    }

    private function UpdateStatusTraking($item = [])
    {
        if (!in_array($item['status'], ConfigStatus::$skip_status) && (int)$item['cancel_order'] === 0) {

            if (round((time() - strtotime($item['status_date'])) / (60 * 60)) >= ConfigStatus::$status_interval) {

                $orderstatustraking = new OrderStatusTraking();
                $orderstatustraking->setIdOrderPresta((int)$item['id_order_presta']);
                $orderstatustraking->setNumberOrder($item['number_order']);
                $orderstatustraking->setOrderId((int)$item['order_id']);
                $status = $orderstatustraking->UpdateStatus();
                if (!is_null($status)) {
                    $item['status'] = $status;
                }
            }
        }
        $item['status'] = ApiUtils::getStatusText($item['status']);
        return $item;
    }

    public function getOrderById(int $id): ?array
    {
        return OrdersModel::getOrderById($id);
    }

    public function setCanceled(int $id)
    {
        return OrdersModel::CanceledOrder($id);
    }
}